/* Full Color LED Test-Program
   Program By GucciTech - Chojiro Higashiguchi
   Date 2008/06/16
   Program Name 'I2C_RTC_V2.C'
   PIC MPU PIC16F877-20/SP
   LCD = SCL1602BSLB-SO-G
   RTC-Chip = RTC-8564NB
   
   I/O Pin Layout
      PORT-A0     Edit-SW
      PORT-A1     Up-SW
      PORT-A2     Down-SW
      PORT-A3     Right-SW
      PORT-A4     Left-SW
      PORT-A5     Ent-SW

      PORT-C3     I2C-SCL
      PORT-C4     I2C-SDA

      PORT-D0     LCD-E
      PORT-D1     LCD-RS
      PORT-D2     LCD-R/W
      PORT-D3     N.C
      PORT-D4     LCD-DB0
      PORT-D5     LCD-DB1
      PORT-D6     LCD-DB2
      PORT-D7     LCD-DB3
*/

#include "I2C_RTC_V2.h"

#define LCD_mode 0      ////// LCD Disp Library Set
#include "lcd_lib_G1.c"

#include "rtc_lib.c"

void time_disp1();         //Now Date & Time Disp
void cursor_update();      //Cursor Position Update

int   year_h2 = 20;        //Year ??xx Data Buffer
int   sw_temp;             //SW Input Data Buffer
int   UpDn_sw_cnt = 0;     //Up-Dwon Switch Counter Buffer
int   RL_sw_cnt = 0;       //Right-Left Switch Counter Buffer

static char d_week[7][4]=
{
   {"Sun"},
   {"Mon"},
   {"Tue"},
   {"Wed"},
   {"Thu"},
   {"Fri"},
   {"Sat"}
};

void time_disp1()
{
      rtc_date_read();
      lcd_cmd(0x00);    //LCD 1-Line Head
      lcd_cmd(0x02);    //LCD-RAM-Address='0' Cursor=HomePosition
      if ((l_week >= 0x30) && (l_week <= 0x39))
         printf(lcd_data,"%u%c%c/%c%c/%c%c %s",year_h2,h_year,l_year,h_month,l_month,h_day,l_day,d_week[l_week - 0x30]);
      else
         printf(lcd_data,"%u%c%c/%c%c/%c%c %s",year_h2,h_year,l_year,h_month,l_month,h_day,l_day,d_week[l_week - (0x30 + 8)]);
      lcd_cmd(0xC0);    //LCD 2-Line Head
      printf(lcd_data,"%c%c:%c%c:%c%c",h_hour,l_hour,h_min,l_min,h_sec,l_sec);
}

void cursor_update(int Cur_posi)
{
   switch(Cur_posi)
   {
      case 0 : lcd_cmd(0x83); break;   //Year Position
      case 1 : lcd_cmd(0x86); break;   //Month Position
      case 2 : lcd_cmd(0x89); break;   //Day Position
      case 3 : lcd_cmd(0x8d); break;   //Week Position
      case 4 : lcd_cmd(0xc1); break;   //Hour Position
      case 5 : lcd_cmd(0xc4); break;   //Minut Position
      case 6 : lcd_cmd(0xc7); break;   //Second Position
      default : break;
   }
}

void Up_value(int Upval)
{
   switch(Upval)
   {
      case 0:
      {              //Year Valu Data Increment
         year++;
         if ((year & 0x0f) == 0x0a)
            year = ((year + 0x10) & 0xf0);
            if ((year & 0xf0) == 0xa0)
            {
               year = year & 0x0f;
               year_h2++;
            }
         rtc_date_set();      //Update Value Data Set
         break;
      }
      case 1:
      {              //Month Value Data Increment
         month++;
         if ((month & 0x0f) == 0x0a)
            month = ((month + 0x10) & 0xf0);
            if (month >= 0x13 )
               month = 0x01;
         rtc_date_set();      //Update Value Data Set
         break;
      }
      case 2:
      {              //Day Value Data Increment
         day++;
         if ((day & 0x0f) == 0x0a)
            day = ((day + 0x10) & 0xf0);
         switch(month)
         {
            case 0: break;
            case 1:
            {
               if (day == 0x32)
                  day = 0x01;
               break;
            }
            case 2:
            {
               if (day == 0x30)
                  day = 0x01;
               break;
            }
            case 3:
            {
               if (day == 0x32)
                  day = 0x01;
               break;
            }
            case 4:
            {
               if (day == 0x31)
                  day = 0x01;
               break;
            }
            case 5:
            {
               if (day == 0x32)
                  day = 0x01;
               break;
            }
            case 6:
            {
               if (day == 0x31)
                  day = 0x01;
               break;
            }
            case 7:
            {
               if (day == 0x32)
                  day = 0x01;
               break;
            }
            case 8:
            {
               if (day == 0x32)
                  day = 0x01;
               break;
            }
            case 9:
            {
               if (day == 0x31)
                  day = 0x01;
               break;
            }
            case 10:
            {
               if (day == 0x32)
                  day = 0x01;
               break;
            }
            case 11:
            {
               if (day == 0x31)
                  day = 0x01;
               break;
            }
            case 12:
            {
               if (day == 0x32)
                  day = 0x01;
               break;
            }
            default : break;
         }
         rtc_date_set();      //Update Value Data Set
         break;
      }
      case 3:
      {              //Week Value Data Increment
         week++;
         if (week >= 0x07 )
            week = 0;
         rtc_date_set();      //Update Value Data Set
         break;
      }
      case 4:
      {              //Hour Value data Increment
         hour++;
         if ((hour & 0x0f) == 0x0a)
            hour = ((hour + 0x10) & 0xf0);
            if (hour >= 0x24 )
               hour = 0x00;
         rtc_date_set();      //Update Value Data Set
         break;
      }
      case 5:
      {              //Minut Value Data Increment
         min++;
         if ((min & 0x0f) == 0x0a)
            min = ((min + 0x10) & 0xf0);
            if (min >= 0x60 )
               min = 0x00;
         rtc_date_set();      //Update Value Data Set
         break;
      }
      case 6:
      {              //Second Value Data Increment
         sec++;
         if ((sec & 0x0f) == 0x0a)
            sec = ((sec + 0x10) & 0xf0);
            if (sec >= 0x60 )
               sec = 0x00;
         rtc_date_set();      //Update Value Data Set
         break;
      }
      default : break;
   }
}

void Down_value(int Dnval)
{
   switch(Dnval)
   {
      case 0:
      {              //Year Valu Data Decrement
         year--;
         if (year == 0xff)
         {
            year = 0x99;
            year_h2--;
         }
         if ((year & 0x0f) == 0x0f)
            year = ((year & 0xf0) | 0x09);
         rtc_date_set();      //Update Value Data Set
         break;
      }
      case 1:
      {              //Month Value Data Decriment
         month--;
         if (month == 0x00 )
            month = 0x12;
         if ((month & 0x0f) == 0x0f)
            month = ((month & 0xf0) | 0x09);
         rtc_date_set();      //Update Value Data Set
         break;
      }
      case 2:
      {              //Day Value Data Decriment
         day--;
         switch(month)
         {
            case 0: break;
            case 1:
            {
               if (day == 0)
                  day = 0x31;
               break;
            }
            case 2:
            {
               if (day == 0)
                  day = 0x29;
               break;
            }
            case 3:
            {
               if (day == 0)
                  day = 0x31;
               break;
            }
            case 4:
            {
               if (day == 0)
                  day = 0x30;
               break;
            }
            case 5:
            {
               if (day == 0)
                  day = 0x31;
               break;
            }
            case 6:
            {
               if (day == 0)
                  day = 0x30;
               break;
            }
            case 7:
            {
               if (day == 0)
                  day = 0x31;
               break;
            }
            case 8:
            {
               if (day == 0)
                  day = 0x31;
               break;
            }
            case 9:
            {
               if (day == 0)
                  day = 0x30;
               break;
            }
            case 10:
            {
               if (day == 0)
                  day = 0x31;
               break;
            }
            case 11:
            {
               if (day == 0)
                  day = 0x30;
               break;
            }
            case 12:
            {
               if (day == 0)
                  day = 0x31;
               break;
            }
            default : break;
         }
         if ((day & 0x0f) == 0x0f)
            day = ((day & 0xf0) | 0x09);
         rtc_date_set();      //Update Value Data Set
         break;
      }
      case 3:
      {              //Week Value Data Decriment
         week--;
         if (week == 0xff )
            week = 0x06;
         rtc_date_set();      //Update Value Data Set
         break;
      }
      case 4:
      {              //Hour Value data Decriment
         hour--;
         if (hour == 0xff)
            hour = 0x23;
         if ((hour & 0x0f) == 0x0f)
            hour = ((hour & 0xf0) | 0x09);
         rtc_date_set();      //Update Value Data Set
         break;
      }
      case 5:
      {              //Minut Value Data Decriment
         min--;
         if (min == 0xff)
            min = 0x59;
         if ((min & 0x0f) == 0x0f)
            min = ((min & 0xf0) | 0x09);
         rtc_date_set();      //Update Value Data Set
         break;
      }
      case 6:
      {              //Second Value Data Up
         sec--;
         if (sec == 0xff)
            sec = 0x59;
         if ((sec & 0x0f) == 0x0f)
            sec = ((sec & 0xf0) | 0x09);
         rtc_date_set();      //Update Value Data Set
         break;
      }
      default : break;
   }
}

//#define Edit_SW   PIN_A0
//#define Ent_SW    PIN_A5

void main()
{
   setup_adc_ports(NO_ANALOGS);
   setup_adc(ADC_OFF);
   setup_psp(PSP_DISABLED);
   setup_timer_0(RTCC_INTERNAL|RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);
   setup_timer_2(T2_DISABLED,0,1);

   // TODO: USER CODE!!

   // LCD Initialize //
   lcd_init();          //LCD Initialize
   lcd_cmd(0x0C);       //LCD Cursor OFF
   lcd_clear();         //LCD ALL CLEAR

   // RTC Vdd Voltage Down Check! //
   i2c_start();
   i2c_write(0xa2); // Wite Mode
   i2c_write(0x02); // Second Address
   i2c_start();
   i2c_write(0xa3); // Read Mode
   sec=  i2c_read(1); // Second Value
   i2c_stop();
   if (sec & 0b10000000)   //VL-bit='1'?
   {
      year_h2 = 20;        //First Data set
      year=0x08; month=0x86; week=0x02; day=0x17; hour=0x12; min=0x00; sec=0x00;
      rtc_control_init();  //RTC-8564NB Initialize
      rtc_date_set();      //First Value Data Set
      rtc_date_read();
   }

   while(True)
   {
      time_disp1();     //Now Date & Time Display!

      if (~input(Edit_SW) == 1)
      {
         delay_ms(10);                 //Chataring Cnacel Delay Time
         //sw_temp = (~input(Edit_SW));
         if (~input(Edit_SW) == 1)
         {
            lcd_cmd(0x0F);          //display ON,Cursor ON,Blink ON
            lcd_cmd(0x83);          //LCD-RAM-Address='3'
            UpDn_sw_cnt = 0;        //Up-Down Switch Counter Buffer Clear
            RL_sw_cnt = 0;          //Right-Left Switch Counter Buffer Clear
            while(True)
            {
               if ((~input_a() & 0b00011110) != 0)
               {
                  sw_temp = (~input_a() & 0b00011110);
                  delay_ms(10);              //Chataring Cnacel Delay Time
                  if ((~input_a() & 0b00011110) == sw_temp)
                  {
                     switch(sw_temp)
                     {
                        case(0b00001000):    //Right-Switch='ON'?
                        {
                           RL_sw_cnt++;
                           if (RL_sw_cnt <= 6)
                           {
                              cursor_update(RL_sw_cnt);
                           }
                           else
                           {
                              RL_sw_cnt = 0;
                              cursor_update(RL_sw_cnt);
                           }
                           break;
                        }
                        case(0b00010000):    //Left-Switch='ON'?
                        {
                           if (RL_sw_cnt == 0)
                           {
                              RL_sw_cnt = 6;
                              cursor_update(RL_sw_cnt);
                           }
                           else
                           {
                              RL_sw_cnt--;
                              cursor_update(RL_sw_cnt);
                           }
                           break;
                        }
                        case(0b00000010):    //Up-Switch='ON'?
                        {
                           Up_value(RL_sw_cnt);
                           time_disp1();     //Now Date & Time Display!
                           lcd_cmd(0x0F);    //display ON,Cursor ON,Blink ON
                           cursor_update(RL_sw_cnt);
                           break;
                        }
                        case(0b00000100):    //Down-Switch='ON'?
                        {
                           Down_value(RL_sw_cnt);
                           time_disp1();     //Now Date & Time Display!
                           lcd_cmd(0x0F);    //display ON,Cursor ON,Blink ON
                           cursor_update(RL_sw_cnt);
                           break;
                        }
                        default: break;
                     }
                     While ((~input_a() & 0b00011110) != 0){ //ALL Switch 'OFF'?
                     }
                     delay_ms(10);           //Chataring Cnacel Delay Time
                  }
               }
               if (~input(Ent_SW) == 1)
               {
                  delay_ms(10);              //Chataring Cnacel Delay Time
                  //sw_temp = (~input(Ent_SW));
                  if (~input(Ent_SW) == 1)
                  {
                     lcd_cmd(0x00);          //LCD 1-Line Head
                     lcd_cmd(0x0C);          //display ON,Cursor OFF,Blink OFF
                     break;
                  }
               }
            }
         }
      }
   }
}
