/*RTC Library (rtc_lib.c)

 I2C RTC EPSON RTC-8564 functions  By H.machida@MNCT-S 2003/10/9
 rtc_lib.c

 global variables
*/

int   year,month,week,day,hour,min,sec;                  // Now Time
char  h_year,h_month,h_week,h_day,h_hour,h_min,h_sec;    // Now Time Character 2Place
char  l_year,l_month,l_week,l_day,l_hour,l_min,l_sec;    // Now Time Character 1Place

// <1>Date & Time Setting
void rtc_date_set()
{
    i2c_start();
    i2c_write(0xa2); // Wite Mode
    i2c_write(0x02); // Second Address
    i2c_write(sec);  // Second Value 0-59
    i2c_write(min);  // Minute Value 0-59
    i2c_write(hour); // Hour Value 0-23
    i2c_write(day);  // Day Value 1-31
    i2c_write(week); // Week Value Mon,Tue,Wen,Thu,Fri,Sat,San,0123456 Upper-5bitDon't Car
    i2c_write(month);// Month Value (C:MSB)1-12 C=1 at 21century
    i2c_write(year); // Year Value 00-99
    i2c_stop();
}

// <2>Date & Time Read
void rtc_date_read()
{
    i2c_start();
    i2c_write(0xa2); // Wite Mode
    i2c_write(0x02); // Second Address
    i2c_start();
    i2c_write(0xa3); // Read Mode
    sec = i2c_read(1); // Second Value
    min = (i2c_read(1)) & 0x7f; // Minute Value
    hour = (i2c_read(1)) & 0x3f; // Hour Value
    day = (i2c_read(1)) & 0x3f; // Day Value
    week = (i2c_read(1)) & 0x07; // Week Value
    month = (i2c_read(1)) & 0x9f; // Month Value
    year= i2c_read(0); // Year Value
    i2c_stop();
    h_sec = (( sec >> 4) & 0x07) | 0x30; l_sec = ( sec & 0x0f) | 0x30; 
    h_min = (( min >> 4) & 0x07) | 0x30; l_min = ( min & 0x0f) | 0x30; 
    h_hour = (( hour >> 4) & 0x03) | 0x30; l_hour = ( hour & 0x0f) | 0x30; 
    h_day = (( day >> 4) & 0x03) | 0x30; l_day = ( day & 0x0f) | 0x30; 
    h_week = 0; l_week= ( week & 0x0f) | 0x30; 
    h_month = ((month >> 4) & 0x01) | 0x30; l_month = (month & 0x0f) | 0x30; 
    h_year = (( year >> 4) & 0x0f) | 0x30; l_year = ( year & 0x0f) | 0x30; 
}

// <3>Alarm Set 10-minute Alarm Signal Out
void rtc_alarm_set(int aweek,int aday,int ahour,int amin)
{
    i2c_start();
    i2c_write(0xa2);  // Wite Mode
    i2c_write(0x09);  // Alarm Minute Address
    i2c_write(amin);  // Alarm Minute Value 0-59 MSB='AE'
    i2c_write(ahour); // Alarm Hour Value 0-23
    i2c_write(aday);  // Alarm Day Value 1-31
    i2c_write(aweek); // Alarm Week Value Mon,Tue,Wen,Thu,Fri,Sat,San,0123456 Upper-5bitDon't Car
    i2c_stop();
}

// <4>Timer Set , Timer='1Hz'
void rtc_timer_set(int timer)
{
   i2c_start();
   i2c_write(0xa2); // Wite Mode
   i2c_write(0x0E); // Timer Control Value Address
   i2c_write(0x02); // TE=0,TD1=1,TD0=0 -> 1Hz
   i2c_write(timer); // Timer Value unsigned char
   i2c_stop();
}

// <5>Timer Start
// tc='0x80'->4096Hz,'0x81'->64Hz,'0x82'->1sec,'0x83'->1minute
void rtc_timer_start(int tc)
{
   i2c_start();
   i2c_write(0xa2);  // Wite Mode
   i2c_write(0x0E);  // Timer Control Value Address
   i2c_write(tc);    // Timer Control Value
   i2c_stop();
}

// <6>Frequency Set , Frequency ='1Hz'
// tfreq='0x80'->32.768kHz,'0x81'->1024Hz,'0x82'->32Hz,'0x83'->1Hz
void rtc_clockout_set(int tfreq)
{
    i2c_start();
    i2c_write(0xa2);  // Wite Mode
    i2c_write(0x0D);  // Clock Out Frequency Address
    i2c_write(tfreq); // Clock Out Frequency Value
    i2c_stop();
}

// <7>RTC Control Get Function
// get control reg1
int rtc_control_get()
{
   int mode;
   i2c_start();
   i2c_write(0xa2);  // Wite Mode
   i2c_write(0x01);  // control-Register2 Address
   i2c_start();
   i2c_write(0xa3);  // Read Mode
   mode = i2c_read(0);
   i2c_stop();
   return mode;
}

// <8>Control_Set Function
//0bit:Timer Enable 1bit:Alarm Enable
//2bit:TimerFlag 3bit:Alarm Flag
//4bit:if set to 1, reload timer value after timer event
void rtc_control_set(int mode)
{
   i2c_start();
   i2c_write(0xa2);  // Wite Mode
   i2c_write(0x01);  // control-Register2 Address
   i2c_write(mode);  // Mode Set
   i2c_stop();
}

// <9>RTC Initialize Function
// set control reg0
void rtc_control_init()
{
   delay_ms(700);
   i2c_start();
   i2c_write(0xa2);  // Wite Mode
   i2c_write(0x00);  // control-Register1 Address
   i2c_write(0x0);   // test=0
   i2c_write(0x0);   // AIE=TIE=0
   i2c_stop();
}
